"""
made by : Matthew R and William L

////////////////////////////
-----welcome to skyport-----
////////////////////////////

    the purpos of skyport is to:
provide a simpler way to make a 2d game with pygame without having to learn how to use pygame itself.

engine classes:
- Display_manager: main display manager class
- Delta_timer: delta time couculator
- Util: utility functions
- Layar_manager: manager for multiple layers (cameras)
- Camera: layer (camera) class
- r_obj: renderable object class
- Loader: asset loader class
- loger: logging utility
- Sprite: animated texture class

"""



"""
class Util: this class is just for utilaty methods and theses are the methods it has
    disp_text(text, font, color, x, y,display): this method displays text (u need a pygame font),
    output_print_data(): this prints evrything in the print que,
    print(string): this puts a string in the print que,
    get_angle_and_dist(self,x1,y1,x,y): this rets the angle and dist (in that order) between 2 points,
    color_swap(self,surface: 'pygame.Surface', old_color: tuple, new_color: tuple) -> 'pygame.Surface': this method swaps evry
        instance of on ecolor on a pygame surf with a nuther,
    pryoraty(self,a=None,b=None): this will alwas return a if there is an a i know aaaaaaaa thats scary,
    sort_objects_by_attr(self,obj_list : list, attr_name : str, reverse=False): this sorts a list of objs by an attar,

class Loader: this class is responsable for file management of all pygame suported images adio and json files
    load_texture_map(self,map_path): this you pass in a path to a texture map and it loades it,
    load_sound_map(self,map_path): this you pass in a path to a sound map and it loades it,
    load_file_map(self,map_path): this you pass in a path to a file map and it loades it,
    NOTE: the next 3 methods all return stuff loaded and prechashed by pygame and json loaders,
    image(path): this returns the image at the path,
    data(path): this returns the json at the path,
    sound(path): this returns the sound at the path,
    warp_image(self,image,sizex,sizey,angle): this returns your image exept warped to the specifacations,
    rotate_image(self,image,angle): this rotates your image,
    scale_image(self,image,sizex,sizey): this scales your image ,
    play_sound(self,file_path, volume=0.5,loops=0): this actualy plays a sound

class Delta_timer: this class gets the dt
    def get_dt(): this returns the delta time
"""

import os
import sys  
BASE_DIR = os.path.dirname(os.path.abspath(__file__))
if BASE_DIR not in sys.path:
   sys.path.insert(0, BASE_DIR)

#from rendering_eng import *
#from global_utils import *
#from assets.layar_manager import *

from .rendering_eng import (Display_manager,GSTI)

from .global_utils import (
    Delta_timer, Util, Loader, r_obj, Sprite,util,loger
)
from .core.paths import pygame

prin_RED = '\033[91m'
prin_GREEN = '\033[92m'
prin_BLUE = '\033[94m'
prin_RESET = '\033[0m'
print_YELLOW = '\033[33m'
print_MAGENTA = '\033[35m'
print_CYAN = '\033[36m'
prin_ORANGE = '\033[38;5;208m'
prin_PINK = '\033[38;5;206m'
prin_PURPLE = '\033[38;5;129m'
prin_BROWN = '\033[38;5;94m'
prin_GOLD = '\033[38;5;220m'
prin_LIME = '\033[38;5;118m'
prin_TEAL = '\033[38;5;30m'
prin_NAVY = '\033[38;5;18m'
prin_SKY_BLUE = '\033[38;5;117m'
prin_HOT_PINK = '\033[38;5;198m'
prin_MAROON = '\033[38;5;88m'
prin_OLIVE = '\033[38;5;100m'
prin_VIOLET = '\033[38;5;93m'
prin_SALMON = '\033[38;5;209m'
prin_DARK_GREEN = '\033[38;5;22m'

from .assets.layar_manager import Layar_manager
from .assets.layar_manager import Camera
from .gf_map_cunstructor import gen_map



import datetime
import os
import json
from pathlib import Path
import pygame
pygame.init()

ENGINE_ROOT = Path(__file__).resolve().parent.parent

class PathUtil:
    @staticmethod
    def fp(relative_path: str) -> Path:
        p = Path(relative_path)
        if p.is_absolute():
            return p
        return (ENGINE_ROOT / p).resolve()

class ENG_Loger:
    ENGINE_LOG_PATH = PathUtil.fp("logs/engine_logs.json")
    if not os.path.exists(ENGINE_LOG_PATH):
        with open(ENGINE_LOG_PATH,"w") as f:
            json.dump({"total_logs": 5,"record_engine_logs": True,"logs": []},indent=4)
    with open(ENGINE_LOG_PATH,"r") as f:
        log_file = json.load(f)
    take_logs = log_file["record_engine_logs"]
    log_data_retaintion_amount = 5
    print(f"--engine logging is set to {take_logs}--")
    def __init__(self):
        self.log_start_time = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        self.run_time_log = ""
        if ENG_Loger.take_logs:
            self.log = self._true_log
        else:
            self.log = self._false_log

    def save(self):
        ENG_Loger.log_file["logs"].append({
            "log_start_time":self.log_start_time,
            "log_end_time":datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
            "run_time_log":self.run_time_log
        })
        if len(ENG_Loger.log_file["logs"]) > ENG_Loger.log_data_retaintion_amount:
            ENG_Loger.log_file["logs"].pop(0)
            ENG_Loger.log_file["total_logs"] = ENG_Loger.log_data_retaintion_amount
        with open(ENG_Loger.ENGINE_LOG_PATH,"w") as f:
            json.dump(ENG_Loger.log_file,f,indent=4)
    def print(self):
        for log in ENG_Loger.log_file["logs"]:
            for line in log["run_time_log"].split("\n"):
                print(line)
            print("----- end of log")

    def _true_log(self,message):
        self.run_time_log += f"[{datetime.datetime.now().strftime('%H:%M:%S')}] :-: {message}\n"
    def _false_log(self,message):
        pass
    def get_logs(self):
        return ENG_Loger.log_file["logs"]

loger = ENG_Loger()



import os
import json

def main():
    base_dir = input("enter the file dir : ").strip()
    base_dir = os.path.abspath(base_dir)

    count = int(input("how meny file formats to use : "))
    extensions = []

    for i in range(count):
        ext = input("file type : ").strip().lower()
        if not ext.startswith("."):
            ext = "." + ext
        extensions.append(ext)

    print(f"\nScanning directory: {base_dir}")
    print(f"Including file types: {extensions}\n")

    manifest = {}

    for root, _, files in os.walk(base_dir):
        for file in files:
            if any(file.lower().endswith(ext) for ext in extensions):
                full_path = os.path.join(root, file)
                rel_path = os.path.relpath(full_path, base_dir)
                rel_path = rel_path.replace("\\", "/")  # engine-safe

                manifest[rel_path] = {
                    "type": "d",
                    "value": None
                }

    print("--- GENERATED ASSET MANIFEST (JSON) ---")
    print(json.dumps(manifest, indent=4))
    """
    save = input("\nSave to file? (y/n): ").lower()
    if save == "y":
        out_name = input("Output file name (ex: assets.json): ").strip()
        with open(out_name, "w", encoding="utf-8") as f:
            json.dump(manifest, f, indent=4)
        print(f"Saved to {out_name}")
    """

    #input("\npress any key to exit:")
def gen_map():
    main()

if __name__ == "__main__":
    main()
    confirm = input("Press Enter to exit...")




from setuptools import setup, find_packages

setup(
    name="skyport_engine",
    # version format: major version . minor version . patch version
    version="0.1.27",
    include_package_data=True,
    package_data={"skyport": ["*.json", "**/*.json", "*.png", "**/*.png", "*.mp3", "**/*.mp3"]},
    install_requires=[
        'pygame-ce>=2.5.5',
        'numpy>=2.3.2'
        ]
    )
#C:\Users\matth\OneDrive\Desktop\skyport
#C:\Users\matth\OneDrive\Desktop\skyport\dist\skyport-0.1.0-py3-none-any.whl

#compile with:
# cd "C:\Users\matth\OneDrive\Desktop\skyport" && python setup.py sdist bdist_wheel
#pip install dist\skyport_engine-0.1.26-py3-none-any.whl

#twine upload dist/*